############################
#
# WITHIN DATASET
#
############################
#
# Fields
# - Naming
# -

# Data sets


# Field naming
# Data set naming
# Names should mean the same thing
# Type consistency
# Documentation length
